package com.winhc.repal.cloud;

import com.winhc.common.base.ApiDataResult;
import com.winhc.common.model.base.VOPage;
import com.winhc.repal.cloud.dto.*;
import com.winhc.repal.cloud.vo.*;
import io.swagger.annotations.ApiOperation;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;


/**
 * @Description: DataMiddlewareService
 * @Author: xda
 * @Date: 2022/4/14 14:21
 */

@FeignClient("winhc-data-middleware")
//@FeignClient(value = "winhc-data-middleware", url = "http://47.101.212.122:10001/")
public interface DataMiddlewareService {




    /**
     * 财产线索数量
     * @param dto dto
     * @return com.winhc.common.base.ApiDataResult<java.util.List < FinanceDynamicNumVO>>
     * @author xda
     * @date 2022/4/14 14:22
     */
    @PostMapping("/dynamic/finance/v8/num")
    ApiDataResult<List<FinanceDynamicNumVO>> getFinanceDynamicNumV8List(@RequestBody GetFinanceDynamicNumDTO dto);


    /**
     * 获取公司风险数量
     * @param dto dto
     * @return com.winhc.common.base.ApiDataResult<java.util.List < com.winhc.repal.cloud.vo.CompanyRiskNumVO>>
     * @author xda
     * @date 2022/4/14 14:41
     */
    @PostMapping("/risk/companyList")
    ApiDataResult<List<CompanyRiskNumVO>> getCompanyRiskNumList(@RequestBody GetCompanyRiskNumDTO dto);

    /**
     * 分页查询v8索引
     * @param dto
     * @param pageNum
     * @param pagSize
     * @return
     */
    @GetMapping("/dynamic/v8/page")
    ApiDataResult<VOPage<V8DynamicVO>> pageDynamic(QueryV8DynamicDTO dto,
                                                   @RequestParam(defaultValue = "1") Integer pageNum,
                                                   @RequestParam(defaultValue = "10") Integer pagSize);

    /**
     * 查找财产线索动态
     * @param dto
     * @return
     */
    @PostMapping("/dynamic/finance/v8/page")
    @ApiOperation("企业财产线索动态 分页")
    ApiDataResult<VOPage<FinanceDynamicVO>> pageFinanceDynamic(@RequestBody QueryESFinanceDynamicDTO dto);


    /**
     * companyIdList失信限高label
     * @param dto dto
     * @return com.winhc.common.base.ApiDataResult<java.util.List < com.winhc.repal.cloud.vo.CreditPunishmentLabelListVO>>
     * @author xda
     * @date 2022/5/5 09:23
     */
    @PostMapping("/credit/label")
    ApiDataResult<List<CreditPunishmentLabelListVO>> getCreditPunishmentLabelList(@RequestBody GetCreditPunishmentLabelDTO dto);
}
