package com.winhc.repal.repository;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.winhc.repal.entity.RepalBill;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.winhc.repal.model.bo.ManageBillInfoBO;
import com.winhc.repal.model.bo.RepalManageBillPageBO;
import com.winhc.repal.model.bo.TotalMoneyBO;
import com.winhc.repal.model.dto.GetRepalBillPageDTO;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.time.LocalDate;
import java.time.LocalDateTime;

/**
 * @description RepalBill Mapper 接口
 * @author Generator
 * @date 2022-04-08
 */
@Repository
public interface RepalBillMapper extends BaseMapper<RepalBill> {


    /**
     * 获取账款列表
     * @param page page
     * @param dto dto
     * @return com.baomidou.mybatisplus.extension.plugins.pagination.Page<com.winhc.repal.entity.RepalBill>
     * @author xda
     * @date 2022/4/13 17:44
     */
    Page<RepalBill> getRepalBillPage(Page<RepalBill> page, GetRepalBillPageDTO dto);


    /**
     * 获取对应类型的总金额和笔数
     * @param repalGroupId repalGroupId
     * @param repalGroupType repalGroupType
     * @param repalBillStatus repalBillStatus
     * @param repalBookId repalBookId
     * @return com.winhc.repal.model.bo.TotalMoneyBO
     * @author xda
     * @date 2022/4/30 09:25
     */
    TotalMoneyBO getTotalMoneyByGroup(@Param("repalGroupId") Long repalGroupId,
                                      @Param("repalGroupType") Integer repalGroupType,
                                      @Param("repalBillStatus") Integer repalBillStatus,
                                      @Param("repalBookId") Long repalBookId);

    /**
     * 获取用户所负责的账款信息金额和count
     * @param userId userId
     * @param repalGroupId repalGroupId
     * @param repalGroupType repalGroupType
     * @param repalBillStatus repalBillStatus
     * @param repalBookId repalBookId
     * @return com.winhc.repal.model.bo.TotalMoneyBO
     * @author xda
     * @date 2022/4/29 14:19
     */
    TotalMoneyBO getTotalMoneyByUser(@Param("userId") Long userId,
                                     @Param("repalGroupId") Long repalGroupId,
                                     @Param("repalGroupType") Integer repalGroupType,
                                     @Param("repalBillStatus") Integer repalBillStatus,
                                     @Param("repalBookId") Long repalBookId);

    /**
     * 获取账款列表
     * @param page page
     * @param custId custId
     * @return com.baomidou.mybatisplus.extension.plugins.pagination.Page<com.winhc.repal.entity.RepalBill>
     * @author xda
     * @date 2022/4/13 17:44
     */
    Page<RepalBill> getCustRepalBillPage(Page<RepalBill> page, @Param("custId") Long custId);

    /**
     * 查询催款汇总数据
     * @param allFlag
     * @param userId
     * @param groupId
     * @return
     */
    ManageBillInfoBO getManageBillInfo(@Param("allFlag") boolean allFlag,
                                       @Param("userId") Long userId,
                                       @Param("groupId") Long groupId,
                                       @Param("startTime") LocalDate startTime);

    /**
     * 分页查询催款列表
     * @param page
     * @param allFlag
     * @param userId
     * @param groupId
     * @param startTime
     * @return
     */
    Page<RepalManageBillPageBO> getManageBillPage(Page<RepalManageBillPageBO>page,
                                                  @Param("allFlag") boolean allFlag,
                                                  @Param("userId") Long userId,
                                                  @Param("groupId") Long groupId,
                                                  @Param("startTime") LocalDate startTime);
}
